from bolinette.defaults.controllers.static import StaticController
from bolinette.defaults.controllers.timezone import TimezoneController
from bolinette.defaults.controllers.file import FileController
from bolinette.defaults.controllers.role import RoleController
from bolinette.defaults.controllers.user import UserController
